!
! ridgidrotor --- global variables for the alignmol program
! Christian Buth
! 06 Apr 2007
!
! Version 1.0.0: 06 Apr 2007 (Global data from Robin Santra's alignmol
!       program moved to this module)
!


      module ridgidrotor
      use fortrantl
      implicit none


!
! Variables
!

!       common /laser/ tau,PtoI,PulseEnergy
!       common /molec/ K,M,init,neq1,Kmax1,Mmax1,A,B,Vmat

      ! Type of the rotor:
      ! 1: linear molecule
      ! 2: prolate symmetric top
      integer RotorType
      ! Type of the nuclear statistics:
      ! 1: linear molecule, N2
      ! 2: linear model for CF3Br/CH3I
      ! 3: prolate symmetric top CF3Br/CH3I
      integer nuclstat

      ! Shape of the laser pulse
      ! 1: Gaussian laser pulse
      ! 2: Gaussian laser pulse with two superimposed oscillations
      integer laserpulseshape

      ! Jmax is the maximum J quantum number assumed to be
      ! initially populated.
      ! Kmax is the maximum K quantum number assumed to be
      ! initially populated.
!       integer K, M, init,
      integer neq, Jmax, Kmax, Mmax, numPulse

      ! PolAni: polarizability anisotropy, i.e.,
      ! alpha_par - alpha_per
      ! in bohr^3
      real(dblprec) tau, PtoI, PulseEnergy, RotConstA, RotConstB, PolAni,  &
     &          Nxrays, xrayDelta, xrayDelz2, NtoF, tau_xr, seqInt,        &
     &          homonuclspin

      real(dblprec), allocatable :: Vmat(:,:,:,:),  &
     &          c2tmat(:,:,:,:)
      complex(dblprec), allocatable :: dmcoeff(:,:), work(:,:)

      contains


!
! xrayflux --- x-ray pulse with a Gaussian shape
!

      real(dblprec) function xrayflux(t, txc, tau, Nxrays)
      use mathtl
      implicit none

      real(dblprec), intent(in) :: t, txc, tau, Nxrays

      xrayflux = 2.0D0 * sqrt(log(2.0D0) / M_PI) / tau * Nxrays  &
     &          * exp(-4.0D0 * log(2.0D0) * ((t - txc) / tau)**2)
      end function xrayflux


!
! laserpower --- power distribution of the laser pulse
!

      real(dblprec) function laserpower(t, tau, PulseEnergy)
      use mathtl
      use physconst
      implicit none

      integer i
      real(dblprec), intent(in) :: t, tau, PulseEnergy
      real(dblprec), parameter ::                           &
     &          aux1 = 2.0D0   * sqrt(log(2.0D0) / M_PI),   &
     &          aux2 = 4.0D0   * log(2.0D0),                &
     &          aux3 = 2.0D0   * log(1.0D0 + sqrt(2.0D0)),  &
     &          T1   = 15.0D0  * PicosecondTot0,            &
     &          T2   = 750.0D0 * FemtosecondTot0,           &
     &          a1   = 0.2D0,                               &
     &          a2   = 0.02D0
      real(dblprec) w1, w2, tcenter

!       parameter(aux1 = 0.93943728d0)
!       parameter(aux2 = 2.7725887d0)

      select case (laserpulseshape)
        ! Gaussian laser pulse
        case(1)
          laserpower = PulseEnergy * aux1 * exp(-aux2*(t/tau)**2) / tau
        ! Gaussian laser pulse with rapid oscillations
        case(2)
          w1 = a1 * sin(M_2_PI * t / T1)
          w2 = a2 * sin(M_2_PI * t / T2)
          laserpower = PulseEnergy * aux1 * exp(-aux2*(t/tau)**2) / tau
          laserpower = laserpower * (1.0D0 + w1 + w2)
        ! Laser pulse from Phys. Rev. A 72, 023420 (2005)
        case(3)
          ! Integration over pulse from minus to plus infinity yields
          ! the pulse energy
          laserpower = PulseEnergy * aux3 / (2.0D0 * tau * cosh(aux3*(t/tau))**2)
        ! Sequence of Gaussian laser pulses
        case(4)
          laserpower = 0.0D0
          tcenter = 0.0D0
          do i=1, numPulse
            laserpower = laserpower + PulseEnergy * aux1  &
     &                   * exp(-aux2*((t-tcenter)/tau)**2) / tau
            tcenter = tcenter + seqInt
          end do
        ! Gaussian laser pulses with wiggles
        case(5)
          w1 = a1 * sin(2.0D0 * M_PI * t / T1)
          w2 = a2 * sin(2.0D0 * M_PI * t / T2)
          laserpower = PulseEnergy * aux1 * exp(-aux2*(t/tau)**2) / tau
          laserpower = laserpower * (1.0D0 + w1 + w2)
        case default
          stop 'laserpower: unkown laser pulse shape requested!'
      end select
      end function laserpower
      end module ridgidrotor


!
! statweight --- compute the statistical weight of the |J K M> states
!       in the partition function
!

      real(dblprec) function statweight(angJ, angK, angM)
      use ridgidrotor
      implicit none

      integer, intent(in) :: angJ, angK, angM

      select case (nuclstat)
        ! Homonuclear diatomic molecule, isotopically identical nuclei
        ! Examples: N2 (N: I=1), Br2 (Br: I=3/2)
        case(1)
          if (mod(angJ,2) == 0) then
!             statweight = 2.0D0
            statweight = homonuclspin + 1.0D0
          else
!             statweight = 1.0D0
            statweight = homonuclspin
          end if
        ! Linear model molecule for CF3Br/CH3I
        case(2)
          statweight = 1.0D0
        ! CF3Br, CH3I with three isotopically identical nuclei
        case(3)
          if (mod(angK,3) == 0) then
            statweight = 4.0D0
          else
            statweight = 2.0D0
          end if
        case default
          stop 'statweight: unkown nuclear statistics type!'
      end select
      end function statweight
