!
! mulph --- multiphoton Floquet calculations
! Robin Santra, Christian Buth
! 5 May 2006
!
! Version V1.0.0: 23 Jun 2006 (Namelist support added)
! Version V1.1.0: 17 Nov 2006 (Diagonalization of the Floquet
!       matrix with the Lanczos algorithm)
!


      program mulph
      use mathtl
      use physconst
      use floquetmat
      implicit none

      logical subEgrnd,  &              ! Shift spectrum to put the HOMO to zero
     &        fulldiag                  ! Diagonalize the Hamiltonian fully

      ! Energy of initial valence level
      real(dblprec) Eground
      ! Laser parameters
      real(dblprec) omega, intens
      real(dblprec) cconst

      ! Orbital angular momentum quantum numbers for the initial state
      integer li, mi
      integer init
      ! Range of orbital angular momenta taken into account
      integer lmin, lmax
      ! Number of l blocks
      integer nl
      ! Range of photon blocks taken into account
      integer nphmin, nphmax
      ! Number of photon blocks
      integer nph
      ! Number of atomic eigenvectors per l and photon block
      integer nvec
      ! Occupied orbitals in the s: nocc(0), p: nocc(1), ... shells
      integer, parameter :: maxlocc = 50
      integer nocc(0:maxlocc), nshellocc(0:maxlocc)

      integer dimen, nmain, niter, nvlnc, neigenpole, i, j, k, l, mu, istart(1)
      real(dblprec) admixture, r, Gamma
      real(dblprec), allocatable :: eval(:,:),      &
     &          polest(:)
      complex(dblprec) z
      complex(dblprec), allocatable :: SESWatom(:,:,:),  &
     &          SESdipatm(:,:,:,:),                 &
     &          SESpotatm(:,:,:),                   &
     &          SESfTfatm(:,:,:),                   &
     &          ceval(:,:),                         &
     &          ceig(:)

      namelist/MultiPhoton/ li, mi, lmin, lmax, nphmin, nphmax, nvec,  &
     &          nocc, omega, intens, init, subEgrnd, fulldiag, niter,  &
     &          admixture, Gamma

      write (*,*) 'mulph V1.1.0, Multiphoton Floquet calculations'
      write (*,*) '(c) 2006 Christian Buth, Robin Santra'
      write (*,*)

      ! Default parameters
      li = 0 ; mi = 0
      lmin = 0 ; lmax = 1
      nphmin = 0 ; nphmax = 0
      nvec = 100
      init = 1
      nocc(:) = 0
      subEgrnd = .false.
      fulldiag = .true.
      omega = 0.0D0
      intens = 0.0D0
      nmain = 1         ! Not modified in the current version of mulph
      niter = 100
      admixture = 0.001D0
      Gamma= 0.0D0

      read(*,nml=MultiPhoton)

      nl = lmax - lmin + 1
      nph = nphmax - nphmin + 1

      ! Echo input
      write (*,*) 'Orbital angular momentum quantum numbers for the ', &
     &            'initial one-particle state: li =', li, 'mi =', mi
      write (*,*) 'Range of orbital angular momenta taken into account ', &
     &            'in Floquet calculation;'
      write (*,*) '    s, p, and d orbitals should always be included: lmin =', &
     &            lmin, 'lmax =', lmax
      write (*,*) 'Number of l blocks: nl =', nl
      write (*,*) 'Number of orbitals occupied in the ground state:'
      write (*,*) 'nocc =', (nocc(j), j=lmin, lmax)
      write (*,*) 'Range of photon blocks taken into account: nphmin =', nphmin, &
     &            'nphmax =', nphmax
      write (*,*) 'Number of photon blocks: nph =', nph
      write (*,*) 'Number of atomic eigenvectors per l and photon block: nvec =', nvec

      dimen = nvec * nl * nph
      write (*,*) 'Dimension of Floquet matrix = ', dimen

      write (*,*) 'Laser photon energy in eV', omega
      write (*,*) 'Laser intensity in W / cm^2', intens
      write (*,*) 'Examine the photoabsorption of state', init
      write (*,*) 'Subtract the energy of the state', subEgrnd
      write (*,*) 'Use full (.true.) or iterative diagonalization (.false.)', fulldiag
      if (.not. fulldiag) then
        write (*,*) 'Carrying out ', niter, ' band-Lanczos iterations',  &
     &              'with ', nmain, 'Lanczos vector'
        write (*,*) 'Random number admixture to the Lanczos vector ', admixture
      end if


!ccccc laser pulse parameters cccccccccccccccccccccccccccccccccccccccc
      ! omega: photon energy in eV:
      ! conversion to hartree:
      omega = omega * eVToHartree
      ! intens: intensity in W/cm^2
      ! conversion to atomic units (an atomic unit of intensity
      ! defined as hartree/t0/a0^2=6.43641d15 W/cm^2)
      intens = intens * intensity_SItoAU

      ! Calculation of the coupling constant sqrt(2 pi alpha intens)
      cconst = sqrt(2.0D0 * M_PI * fine_structure_constant * intens)
      write(*,*) 'radiation coupling constant in a.u. = ', cconst

      ! Experimental decay width of the hole in eV
      write(*,*) 'Experimental decay width of the hole ', Gamma, ' eV'
      Gamma = Gamma * eVToHartree

      ! Read atomic one-electron eigenenergies (eval), dipole matrix elements
      ! (Datom), and matrix elements of the CAP operator W (Watom)
      allocate( eval(nvec,lmin:lmax),                      &
     &          ceval(nvec,lmin:lmax),                     &
     &          SESWatom(nvec,nvec,lmin:lmax),             &
     &          SESdipatm(nvec,lmin:lmax,nvec,lmin:lmax),  &
     &          SESpotatm(nvec,nvec,lmin:lmax),            &
     &          SESfTfatm(nvec,nvec,lmin:lmax))
      call readatom(nl, lmin, lmax, nvec, nvlnc, eval, SESdipatm,  &
     &              SESfTfatm, SESpotatm, SESWatom, li, init, Eground)
!                SESWatom(:,:,:) = 0
!                SESpotatm(:,:,:) = 0
!                SESfTfatm(:,:,:) = 0
      write(*,*) 'Number of the S shell of the valence electron, nvlnc =', nvlnc

      ! Recalculate the nocc array after subtraction of the
      ! core orbitals nvlnc
      nshellocc(:) = 0    ! S orbitals are the first orbitals of the periodic table
      nshellocc(1) = 1    ! P orbitals start in the second row of the periodic table
      nshellocc(2) = 2    ! D orbitals start in the third row of the periodic table
      nshellocc(3) = 3    ! F orbitals start in the fourth row of the periodic table
      do l = lmin, lmax
        j = nvlnc - 1 - nshellocc(l)
        if (nocc(l) > 0 .and. j > 0) then
          nocc(l) = nocc(l) - j
          if (nocc(l) < 0) then
            nocc(l) = 0
          end if
        end if
      end do
      write (*,*) 'Occupation pattern in the ground state after subtraction of ',  &
     &  'the core orbitals nvlnc:'
      write (*,*) 'nocc =', (nocc(j), j=lmin, lmax)

      ! Remove transitions between occupied states
      do l = lmin, lmax
        do k = 1, nocc(l)
          do j = lmin, lmax
            do i = 1, nocc(j)
              SESdipatm(i,j,k,l) = (0.0D0, 0.0D0)
              SESdipatm(k,l,i,j) = (0.0D0, 0.0D0)
            end do
          end do
        end do
      end do

      ! Affix the decay width of the K-vacancy Gamma to the
      ! final state Rydberg orbitals
      do l = lmin, lmax
        do i=1, nvec
          ceval(i,l) = cmplx(eval(i,l), 0.0D0, dblprec)
          if (i > nocc(l)) then
            ceval(i,l) = ceval(i,l) - cmplx(0.0D0, Gamma / 2.0D0, dblprec)
          end if
        end do
      end do

      if (.not. subEgrnd) then
        Eground = 0
      end if

      call hamiltonian(dimen, nmain, nphmin, nphmax, lmin, lmax, nvec,   &
     &          mi, cconst, eval, Eground, omega, SESfTfatm, SESpotatm,  &
     &          SESWatom, SESdipatm, ceval)
      deallocate(eval, ceval, SESWatom, SESdipatm, SESpotatm, SESfTfatm)

      ! Diagonalize the Hamiltonian
      if (fulldiag) then
        allocate(ceig(dimen))
        call fulldiagfloquet('N', dimen, ceig)
        neigenpole = dimen
      else
        allocate(ceig(niter), polest(niter))
        neigenpole = niter
      ! Generate cartesian start vectors.  They depend on the
      ! basis set; the (linear) compound indices are given by
      ! idx -> (mu, l, i) with mu = nphmin..nphmax, l = lmin..
      ! lmax, i = 1..nvec.  Then, the linear index idx, which
      ! corresponds to the triple (mu, l, i) is determined by
      !   idx = nl * nvec * (mu - nphmin) + nvec * (l - lmin) + i
      ! nl : number of angular momenta
      ! 1s state in the mu=0 block
!       do j=1, nmain
        mu = 0
        l = li
        i = init
        j = 1
        istart(j) = nl * nvec * (mu - nphmin) + nvec * (l - lmin) + i
      write (*,*) 'Lanczos start vector No ', j, ' cartesian index for (mu, l, i) = (',  &
     &            mu, ',', l, ',', i, ') is ', istart(j)
!       end do
        call lanczosdiagfloquet(ceig, polest, dimen, nmain, niter, istart, admixture)
      end if

      ! Bubble sort the eigenvalues with ascending real part
      do i = 1, neigenpole
        do j = i+1, neigenpole
          if (real(ceig(i)) > real(ceig(j))) then
            z = ceig(i)
            ceig(i) = ceig(j)
            ceig(j) = z
            if (.not. fulldiag) then
              r = polest(i)
              polest(i) = polest(j)
              polest(j) = r
            end if
          end if
        end do
      end do

      ! Dump eigenvalues to disk
      open(unit=10,file='traj.dat', access='sequential',  &
     &     status='replace', form='formatted')

      write(10,*) neigenpole
      write(10,*) 0.0           ! eta is not used because SES CAPs are employed

      do j=1, neigenpole
        if (fulldiag) then
          write(10,*) ceig(j)
        else
          write(10,*) ceig(j), polest(j)
        end if
      end do
      close(unit=10)

      deallocate(ceig)
      if (.not. fulldiag) then
        deallocate(polest)
      end if

      write(*,*)
      write(*,*) 'Honi soit qui mal y pense!'
      end program mulph
